function [trn,sfx,lrs,si,sj] = AOS(s,thresh)
% [trn,sfx,si,sj] = AOS(s,a)
% Sparse Audio Oracle. It also outputs the length of repeated suffix (lrs).  
% Audio Oracle extends the algorithm of Factor Oracle for metric spaces over sequence of vectors s.
% See AO.m for more documentation.
%
% input:
% s - sequence of vectors. time orientation is along columns
% thresh - minimal allowed distance between symbols (default 0.1)
% output:
% trn - transition matrix (forward)
% sfx - suffix vector (backwards)
% lrs = length of repeated suffix
% si,sj -  i and j vectors according to MATLAB sparse matrix represenation


if nargin < 2,
thresh = 0.1;
end

trn = cell(size(s,2)+1,1);
sfx = zeros(1,size(s,2));

progressbar(0);

cnt = 1;
for i = 2:length(s)+1,
    progressbar(i/length(s));
    symbol = s(:,i-1); %symbol is a vector
    trn{i-1} = i; %forward step
    
    si(cnt) = i-1;
    sj(cnt) = i;
    cnt = cnt+1;
    
    k = sfx(i-1);
    pi_1= i-1; %Shlomo 8/28/09 

    while k ~= 0,
        dvec = dts(trn{k},symbol,s);
        I = find(dvec<thresh);
        if isempty(I), %no transition with symbol from state k 
            trn{k} = [trn{k} i]; %append (add) i to set of transitions from state k
   
            pi_1 = k; %Shlomo 8/28/09
           
            si(cnt) = k;
            sj(cnt) = i;
            cnt = cnt+1;

            k = sfx(k); %follow suffix link
            
            
        else
            break
        end
    end
    if k == 0, %if no transition exists
        sfx(i) = 1; %point to state 1 (beginning)
        
        si(cnt) = i;
        sj(cnt) = 1;
        cnt = cnt+1;
        
    else 
        % choose first transition from k that is < thresh as suffix
        %sfx(i) = trn{k}(min(I)); 
        % another option: choose suffix link to state with minimal distance
        [val,ival] = min(dvec(I));
        sfx(i) = trn{k}(I(ival));
        
        si(cnt) = i;
        sj(cnt) = sfx(i);
        cnt = cnt+1; 
        
    end

    % Computing lrs (Lefebvre and Lecroq (2000))
    ss=sfx(i); %Shlomo 11/24/.06 
    if ss == 0 | ss==1 % was only k==0
        lrs(i) = 0;
    else
        pi_2 = ss-1; % was k-1;
        if pi_2 == sfx(pi_1),
            lrs(i) = lrs(pi_1) + 1;
        else
            while sfx(pi_2) ~= sfx(pi_1)
                pi_2 =sfx(pi_2);
            end
            lrs(i) = min(lrs(pi_1),lrs(pi_2)) + 1;
        end
    end

end
%trn{end} = [2:length(s)]; %loop? last state point to any state?
trn{end} = 1; %last state to the begining?

function dist = dts(fwd,symbol,s)
% vector of distances between new symbol and all states 
% pointed to by fwd transition links.
% The distance is normalized by the dimension of the feature vector

D = length(symbol); %dimension of the feature vector

for i = 1:length(fwd),
    dist(i) = norm(s(:,fwd(i)-1)-symbol); %fwd(i)-1 since fwd indexes are shifted by one (start with 1 and not zero like in the original FO).
end

dist = dist/D;